/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gio/gio.h>

#include <user-awareness.h>

#include "uat-menu.h"
#include "uat-common.h"

extern ua_device_h g_device_h; /**< Device handle */
extern ua_service_h g_service_h; /**< Service handle */
ua_payload_h g_payload_h = NULL; /**< Payload handle */

static char g_payload_primary_key[MENU_DATA_SIZE + 1] = {0,}; /**< payload primary_key for the selected device */
static char g_payload_device_icon[MENU_DATA_SIZE + 1] = {0,}; /**< payload device_icon for the selected device */
static char g_payload_secondary_key[MENU_DATA_SIZE + 1] = {0,}; /**< payload secondary_key for the selected device */
static char g_payload_device_uid_len[MENU_DATA_SIZE + 1] = {0,}; /**< payload device_uid len for the selected device */
static char g_payload_device_uid_str[MENU_DATA_SIZE + 1] = {0,}; /**< payload device_uid for the selected device */

GSList *payload_list = NULL; /**< Payload List */
int idx_payload = 0; /**< Found payload sequence number */

char g_selected_payload_idx[MENU_DATA_SIZE + 1] = {0,}; /** Selected payload id */

static void update_payload_info(void)
{
	int ret = UA_ERROR_NONE;
	char primary_key = 0;
	char secondary_key = 0;
	char *device_uid = NULL;
	char device_icon = 0;

	if (0 == idx_payload) {
		if (g_payload_h) {
			ua_payload_destroy(g_payload_h);
			g_payload_h = NULL;
		}
		//TO-DO Add support for default payload
		//ua_payload_get_default_payload(&g_payload_h);
	}

	ret = ua_payload_get_primary_key(g_payload_h, &primary_key);
	if (UA_ERROR_NONE == ret) {
		memset(g_payload_primary_key, 0, MENU_DATA_SIZE + 1);
		memcpy(g_payload_primary_key, &primary_key, sizeof(primary_key));
	}

	ret = ua_payload_get_secondary_key(g_payload_h, &secondary_key);
	if (UA_ERROR_NONE == ret) {
		memset(g_payload_secondary_key, 0, MENU_DATA_SIZE + 1);
		memcpy(g_payload_secondary_key, &secondary_key, sizeof(secondary_key));
	}

	ret = ua_payload_get_device_uid(g_payload_h, &device_uid);
	if (UA_ERROR_NONE == ret) {
		memset(g_payload_device_uid_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_payload_device_uid_str, device_uid, MENU_DATA_SIZE);
		g_free(device_uid);
	}

	ret = ua_payload_get_device_icon(g_payload_h, &device_icon);
	if (UA_ERROR_NONE == ret) {
		memset(g_payload_device_icon, 0, MENU_DATA_SIZE + 1);
		memcpy(g_payload_device_icon, &device_icon, sizeof(device_icon));
	}
}

static void  ___free_payload(gpointer data)
{
	ua_payload_h handle = data;
	ua_payload_destroy(handle);
	handle = NULL;
}

void uat_clear_payload_list(void)
{
	g_slist_free_full(payload_list, ___free_payload);
	payload_list = NULL;
	idx_payload = 0;
}

static int run_ua_payload_create(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_payload_create");

	if (g_payload_h) {
		ua_payload_destroy(g_payload_h);
		g_payload_h = NULL;
	}

	ret = ua_payload_create(&g_payload_h);

	msg(" - ua_payload_create() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_payload_destroy(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_payload_destroy");

	ret = ua_payload_destroy(g_payload_h);
	g_payload_h = NULL;

	msg(" - ua_payload_destroy() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

#ifndef SUPPORT_STRING_DEVICE_UID
static int _scan_payload_device_uid_str(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	int temp = 0;
	int device_uid_len = 0;
	int len = 0;

	msg("Input 17 bytes device_uid, each byte is 0 to 255 value");
	memset(g_payload_device_uid_str, 0, MENU_DATA_SIZE + 1);

	if (strlen(g_payload_device_uid_len))
		len = (unsigned char)strtol(g_payload_device_uid_len, NULL, 10);
	len = len < UA_BLE_PAYLOAD_DEVICE_UID_MAX_LEN ? len : UA_BLE_PAYLOAD_DEVICE_UID_MAX_LEN;
	device_uid_len = len > 0 ? len : 0;

	for (int i = 0; i < device_uid_len; i++) {
		ret = fscanf(stdin, "%d", &temp);
		if (!ret)
			msg(" - fscanf ret: %d", ret);

		g_payload_device_uid_str[i] = (unsigned char)temp;
	}

	return RET_SUCCESS;
}
#endif /* SUPPORT_STRING_DEVICE_UID */

static int run_ua_payload_set_primary_key(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	char primary_key = 0;

	msg("ua_payload_set_primary_key");

	if (strlen(g_payload_primary_key))
		primary_key = (unsigned char)strtol(g_payload_primary_key, NULL, 10);

	ret = ua_payload_set_primary_key(g_payload_h, primary_key);

	msg(" - ua_payload_set_primary_key() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_payload_set_device_icon(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	char device_icon = 0;

	msg("ua_payload_set_device_icon");

	if (strlen(g_payload_device_icon))
		device_icon = (unsigned char)strtol(g_payload_device_icon, NULL, 10);

	ret = ua_payload_set_device_icon(g_payload_h, device_icon);

	msg(" - ua_payload_set_device_icon() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_payload_set_secondary_key(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	char secondary_key = 0;

	msg("ua_payload_set_secondary_key");

	if (strlen(g_payload_secondary_key))
		secondary_key = (unsigned char)strtol(g_payload_secondary_key, NULL, 10);

	ret = ua_payload_set_secondary_key(g_payload_h, secondary_key);

	msg(" - ua_payload_set_secondary_key() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_payload_set_device_uid(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	int device_uid_len = 0;

	msg("ua_payload_set_device_uid");

	if (strlen(g_payload_device_uid_len))
		device_uid_len = (unsigned char)strtol(g_payload_device_uid_len, NULL, 10);

	ret = ua_payload_set_device_uid(g_payload_h,
		device_uid_len, g_payload_device_uid_str);

	msg(" - ua_payload_set_device_uid() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_payload_add(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_payload_add");

	check_if(NULL == g_payload_h);

	ret = ua_payload_add(g_payload_h, g_device_h);

	msg(" - ua_payload_add() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_payload_add_service(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_payload_add_service");

	check_if(NULL == g_payload_h);

	ret = ua_payload_add_service(g_payload_h, g_service_h);

	msg(" - ua_payload_add_service() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static bool __foreach_added_payload_cb(
	ua_payload_h payload_handle, void *user_data)
{
	int ret = UA_ERROR_NONE;
	ua_payload_h handle = NULL;
	char primary_key = 0;
	char secondary_key = 0;
	char *device_uid = NULL;
	int device_uid_len = 0;
	char device_icon = 0;
	char *payload_user_data = NULL;

	if (!payload_handle) {
		msgr("payload_handle is NULL");
		return true;
	}

	ret = ua_payload_clone(&handle, payload_handle);
	if (UA_ERROR_NONE != ret) {
		msgr("ua_payload_clone() result: [0x%X] [%s]",
			ret, uat_get_error_str(ret));
	}

	payload_list = g_slist_append(payload_list, handle);
	idx_payload++;

	ret = ua_payload_get_primary_key(handle, &primary_key);
	if (UA_ERROR_NONE == ret) {
		msglr("[%d]", idx_payload);
		msglr("Payload primary_key : %d", primary_key);
	}

	ret = ua_payload_get_secondary_key(handle, &secondary_key);
	if (UA_ERROR_NONE == ret)
		msglr("Payload secondary key : %d", secondary_key);

	ret = ua_payload_get_device_uid_len(handle, &device_uid_len);
	if (UA_ERROR_NONE == ret)
		msglr("Payload device uid len  %d", device_uid_len);

	ret = ua_payload_get_device_uid(handle, &device_uid);
	if (UA_ERROR_NONE == ret) {
		for (int i = 0; i < device_uid_len; i++)
			msglr("Payload device uid 0x%2.2X", device_uid[i]);
		g_free(device_uid);
	}

	ret = ua_payload_get_device_icon(handle, &device_icon);
	if (UA_ERROR_NONE == ret)
		msglr("Payload device icon : %d", device_icon);

	ret = ua_payload_get_user_data(handle, &payload_user_data);
	if (UA_ERROR_NONE == ret) {
		for (int i = 0; i < 17 - 1 - device_uid_len; i++)
			msglr("Payload user data : 0x%2.2X", payload_user_data[i]);
		g_free(payload_user_data);
	}

	return true;
}

static int run_ua_payload_foreach_added(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_payload_foreach_added");

	uat_clear_payload_list();

	ret = ua_payload_foreach_added(__foreach_added_payload_cb, NULL);

	msg(" - ua_payload_foreach_added() ret: [0x%X] [%s]",
		ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

int uat_select_payload(MManager *mm, struct menu_data *menu)
{
	GSList *iter = payload_list;

	int id = 0;
	int selected_id = 0;

	if (strlen(g_selected_payload_idx))
		selected_id = (unsigned char)strtol(g_selected_payload_idx, NULL, 10);

	if (selected_id <= 0) {
		msg("Please select payload first");
		return RET_SUCCESS;
	}

	while (NULL != iter) {
		GSList *next = iter->next;
		ua_payload_h *handle = iter->data;
		if (handle && ++id == selected_id) {
			g_payload_h = handle; // Make selected payload as a current one
			update_payload_info(); // Update payload info.
			msg("Select payload done");
			return RET_SUCCESS;
		}
		iter = next;
	}

	return RET_SUCCESS;
}

static struct menu_data menu_ua_payload_set_primary_key[] = {
	{ "1", "Primary Key (0 to 255)",
		NULL, NULL, g_payload_primary_key },
	{ "2", "run", NULL,
		run_ua_payload_set_primary_key, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_payload_set_device_icon[] = {
	{ "1", "device_icon (0 to 255)",
		NULL, NULL, g_payload_device_icon },
	{ "2", "run", NULL,
		run_ua_payload_set_device_icon, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_payload_set_secondary_key[] = {
	{ "1", "Secondary Key (0 to 255)",
		NULL, NULL, g_payload_secondary_key },
	{ "2", "run", NULL,
		run_ua_payload_set_secondary_key, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_payload_set_device_uid[] = {
	{ "1", "Device uid len (0 to 17)",
		NULL, NULL, g_payload_device_uid_len },
	{ "2", "DEVICE_UID (Max. 17 bytes)",
#ifdef SUPPORT_STRING_DEVICE_UID
		NULL, NULL, g_payload_device_uid_str },
#else
		NULL, _scan_payload_device_uid_str, NULL },
#endif
	{ "3", "run", NULL,
		run_ua_payload_set_device_uid, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_sel_added_payload[] = {
	{ "1", "Payload list", NULL,
		run_ua_payload_foreach_added, g_selected_payload_idx},
	{ "2", "Apply", NULL, uat_select_payload, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_ua_payloads[] = {
	{ "1", "ua_payload_create",
		NULL, run_ua_payload_create, NULL },
	{ "2", "ua_payload_destroy",
		NULL, run_ua_payload_destroy, NULL },
	{ "3", "ua_payload_set_primary_key",
		menu_ua_payload_set_primary_key, NULL, g_payload_primary_key },
	{ "4", "ua_payload_set_device_icon",
		menu_ua_payload_set_device_icon, NULL, g_payload_device_icon },
	{ "5", "ua_payload_set_secondary_key",
		menu_ua_payload_set_secondary_key, NULL, g_payload_secondary_key },
	{ "6", "ua_payload_set_device_uid",
		menu_ua_payload_set_device_uid, NULL, g_payload_device_uid_str },
	{ "7", ANSI_COLOR_LIGHTGREEN "ua_payload_add" ANSI_COLOR_NORMAL,
		NULL, run_ua_payload_add, NULL },
	{ "8", ANSI_COLOR_LIGHTGREEN "ua_payload_add_service" ANSI_COLOR_NORMAL,
		NULL, run_ua_payload_add_service, NULL },
	{ "9", ANSI_COLOR_LIGHTMAGENTA "ua_payload_foreach_added" ANSI_COLOR_NORMAL,
		menu_sel_added_payload, NULL, NULL },
	{ NULL, NULL, },
};
